@@ -1,15 +1,36 @@
+/*
+ * Copyright 2014 - Present Rafael Winterhalter
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 package net.bytebuddy.description.field;
 
+import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
+import net.bytebuddy.build.CachedReturnPlugin;
 import net.bytebuddy.description.ByteCodeElement;
+import net.bytebuddy.description.DeclaredByType;
 import net.bytebuddy.description.ModifierReviewable;
-import net.bytebuddy.description.NamedElement;
 import net.bytebuddy.description.annotation.AnnotationDescription;
 import net.bytebuddy.description.annotation.AnnotationList;
+import net.bytebuddy.description.type.TypeDefinition;
 import net.bytebuddy.description.type.TypeDescription;
 import net.bytebuddy.matcher.ElementMatcher;
+import net.bytebuddy.utility.nullability.AlwaysNull;
+import net.bytebuddy.utility.nullability.MaybeNull;
 import org.objectweb.asm.Opcodes;
 import org.objectweb.asm.signature.SignatureWriter;
 
+import javax.annotation.Nonnull;
 import java.lang.reflect.Field;
 import java.lang.reflect.GenericSignatureFormatError;
 import java.lang.reflect.Modifier;
@@ -20,16 +41,23 @@ import java.util.List;
  * Implementations of this interface describe a Java field. Implementations of this interface must provide meaningful
  * {@code equal(Object)} and {@code hashCode()} implementations.
  */
-public interface FieldDescription extends ByteCodeElement,
-        ModifierReviewable.ForFieldDescription,
-        NamedElement.WithGenericName,
+public interface FieldDescription extends ModifierReviewable.ForFieldDescription,
+        DeclaredByType.WithMandatoryDeclaration,
+        ByteCodeElement.Member,
         ByteCodeElement.TypeDependant<FieldDescription.InDefinedShape, FieldDescription.Token> {
 
     /**
      * A representative of a field's non-set default value.
      */
+    @AlwaysNull
     Object NO_DEFAULT_VALUE = null;
 
+    /**
+     * {@inheritDoc}
+     */
+    @Nonnull
+    TypeDefinition getDeclaringType();
+
     /**
      * Returns the type of the described field.
      *
@@ -57,7 +85,10 @@ public interface FieldDescription extends ByteCodeElement,
      */
     interface InGenericShape extends FieldDescription {
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
+        @Nonnull
         TypeDescription.Generic getDeclaringType();
     }
 
@@ -66,7 +97,10 @@ public interface FieldDescription extends ByteCodeElement,
      */
     interface InDefinedShape extends FieldDescription {
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
+        @Nonnull
         TypeDescription getDeclaringType();
 
         /**
@@ -74,7 +108,9 @@ public interface FieldDescription extends ByteCodeElement,
          */
         abstract class AbstractBase extends FieldDescription.AbstractBase implements InDefinedShape {
 
-            @Override
+            /**
+             * {@inheritDoc}
+             */
             public InDefinedShape asDefined() {
                 return this;
             }
@@ -86,22 +122,31 @@ public interface FieldDescription extends ByteCodeElement,
      */
     abstract class AbstractBase extends ModifierReviewable.AbstractBase implements FieldDescription {
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public String getInternalName() {
             return getName();
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public String getActualName() {
             return getName();
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public String getDescriptor() {
             return getType().asErasure().getDescriptor();
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
+        @MaybeNull
         public String getGenericSignature() {
             TypeDescription.Generic fieldType = getType();
             try {
@@ -113,30 +158,42 @@ public interface FieldDescription extends ByteCodeElement,
             }
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
+        @SuppressFBWarnings(value = "NP_NULL_ON_SOME_PATH_FROM_RETURN_VALUE", justification = "Assuming declaring type for type member.")
         public boolean isVisibleTo(TypeDescription typeDescription) {
             return getDeclaringType().asErasure().isVisibleTo(typeDescription)
                     && (isPublic()
                     || typeDescription.equals(getDeclaringType().asErasure())
-                    || (isProtected() && getDeclaringType().asErasure().isAssignableFrom(typeDescription))
-                    || (!isPrivate() && typeDescription.isSamePackage(getDeclaringType().asErasure())));
+                    || isProtected() && getDeclaringType().asErasure().isAssignableFrom(typeDescription)
+                    || !isPrivate() && typeDescription.isSamePackage(getDeclaringType().asErasure())
+                    || isPrivate() && typeDescription.isNestMateOf(getDeclaringType().asErasure()));
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
+        @SuppressFBWarnings(value = "NP_NULL_ON_SOME_PATH_FROM_RETURN_VALUE", justification = "Assuming declaring type for type member.")
         public boolean isAccessibleTo(TypeDescription typeDescription) {
             return isPublic()
                     || typeDescription.equals(getDeclaringType().asErasure())
-                    || (!isPrivate() && typeDescription.isSamePackage(getDeclaringType().asErasure()));
+                    || !isPrivate() && typeDescription.isSamePackage(getDeclaringType().asErasure())
+                    || isPrivate() && typeDescription.isNestMateOf(getDeclaringType().asErasure());
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public int getActualModifiers() {
             return getModifiers() | (getDeclaredAnnotations().isAnnotationPresent(Deprecated.class)
                     ? Opcodes.ACC_DEPRECATED
                     : EMPTY_MASK);
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public FieldDescription.Token asToken(ElementMatcher<? super TypeDescription> matcher) {
             return new FieldDescription.Token(getName(),
                     getModifiers(),
@@ -144,13 +201,23 @@ public interface FieldDescription extends ByteCodeElement,
                     getDeclaredAnnotations());
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public SignatureToken asSignatureToken() {
             return new SignatureToken(getInternalName(), getType().asErasure());
         }
 
         @Override
-        public boolean equals(Object other) {
+        @CachedReturnPlugin.Enhance("hashCode")
+        @SuppressFBWarnings(value = "NP_NULL_ON_SOME_PATH_FROM_RETURN_VALUE", justification = "Assuming declaring type for type member.")
+        public int hashCode() {
+            return getDeclaringType().hashCode() + 31 * (17 + getName().hashCode());
+        }
+
+        @Override
+        @SuppressFBWarnings(value = "NP_NULL_ON_SOME_PATH_FROM_RETURN_VALUE", justification = "Assuming declaring type for type member.")
+        public boolean equals(@MaybeNull Object other) {
             if (this == other) {
                 return true;
             } else if (!(other instanceof FieldDescription)) {
@@ -160,12 +227,10 @@ public interface FieldDescription extends ByteCodeElement,
             return getName().equals(fieldDescription.getName()) && getDeclaringType().equals(fieldDescription.getDeclaringType());
         }
 
-        @Override
-        public int hashCode() {
-            return getDeclaringType().hashCode() + 31 * (17 + getName().hashCode());
-        }
-
-        @Override
+        /**
+         * {@inheritDoc}
+         */
+        @SuppressFBWarnings(value = "NP_NULL_ON_SOME_PATH_FROM_RETURN_VALUE", justification = "Assuming declaring type for type member.")
         public String toGenericString() {
             StringBuilder stringBuilder = new StringBuilder();
             if (getModifiers() != EMPTY_MASK) {
@@ -177,6 +242,7 @@ public interface FieldDescription extends ByteCodeElement,
         }
 
         @Override
+        @SuppressFBWarnings(value = "NP_NULL_ON_SOME_PATH_FROM_RETURN_VALUE", justification = "Assuming declaring type for type member.")
         public String toString() {
             StringBuilder stringBuilder = new StringBuilder();
             if (getModifiers() != EMPTY_MASK) {
@@ -207,7 +273,9 @@ public interface FieldDescription extends ByteCodeElement,
             this.field = field;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public TypeDescription.Generic getType() {
             if (TypeDescription.AbstractBase.RAW_TYPES) {
                 return TypeDescription.Generic.OfNonGenericType.ForLoadedType.of(field.getType());
@@ -215,27 +283,39 @@ public interface FieldDescription extends ByteCodeElement,
             return new TypeDescription.Generic.LazyProjection.ForLoadedFieldType(field);
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
+        @CachedReturnPlugin.Enhance("declaredAnnotations")
         public AnnotationList getDeclaredAnnotations() {
             return new AnnotationList.ForLoadedAnnotations(field.getDeclaredAnnotations());
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public String getName() {
             return field.getName();
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
+        @Nonnull
         public TypeDescription getDeclaringType() {
             return TypeDescription.ForLoadedType.of(field.getDeclaringClass());
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public int getModifiers() {
             return field.getModifiers();
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public boolean isSynthetic() {
             return field.isSynthetic();
         }
@@ -255,7 +335,7 @@ public interface FieldDescription extends ByteCodeElement,
         /**
          * The name of the field.
          */
-        private final String fieldName;
+        private final String name;
 
         /**
          * The field's modifiers.
@@ -290,44 +370,55 @@ public interface FieldDescription extends ByteCodeElement,
          * Creates a new latent field description. All provided types are attached to this instance before they are returned.
          *
          * @param declaringType       The declaring type of the field.
-         * @param fieldName           The name of the field.
-         * @param fieldType           The field's modifiers.
-         * @param modifiers           The type of the field.
+         * @param name                The name of the field.
+         * @param fieldType           The type of the field.
+         * @param modifiers           The field's modifiers.
          * @param declaredAnnotations The annotations of this field.
          */
         public Latent(TypeDescription declaringType,
-                      String fieldName,
+                      String name,
                       int modifiers,
                       TypeDescription.Generic fieldType,
                       List<? extends AnnotationDescription> declaredAnnotations) {
             this.declaringType = declaringType;
-            this.fieldName = fieldName;
+            this.name = name;
             this.modifiers = modifiers;
             this.fieldType = fieldType;
             this.declaredAnnotations = declaredAnnotations;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public TypeDescription.Generic getType() {
             return fieldType.accept(TypeDescription.Generic.Visitor.Substitutor.ForAttachment.of(this));
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public AnnotationList getDeclaredAnnotations() {
             return new AnnotationList.Explicit(declaredAnnotations);
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public String getName() {
-            return fieldName;
+            return name;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
+        @Nonnull
         public TypeDescription getDeclaringType() {
             return declaringType;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public int getModifiers() {
             return modifiers;
         }
@@ -368,32 +459,45 @@ public interface FieldDescription extends ByteCodeElement,
             this.visitor = visitor;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public TypeDescription.Generic getType() {
             return fieldDescription.getType().accept(visitor);
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public AnnotationList getDeclaredAnnotations() {
             return fieldDescription.getDeclaredAnnotations();
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
+        @Nonnull
         public TypeDescription.Generic getDeclaringType() {
             return declaringType;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public int getModifiers() {
             return fieldDescription.getModifiers();
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public String getName() {
             return fieldDescription.getName();
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public InDefinedShape asDefined() {
             return fieldDescription.asDefined();
         }
@@ -486,7 +590,9 @@ public interface FieldDescription extends ByteCodeElement,
             return new AnnotationList.Explicit(annotations);
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public Token accept(TypeDescription.Generic.Visitor<? extends TypeDescription.Generic> visitor) {
             return new Token(name,
                     modifiers,
@@ -505,7 +611,17 @@ public interface FieldDescription extends ByteCodeElement,
         }
 
         @Override
-        public boolean equals(Object other) {
+        @CachedReturnPlugin.Enhance("hashCode")
+        public int hashCode() {
+            int result = name.hashCode();
+            result = 31 * result + modifiers;
+            result = 31 * result + type.hashCode();
+            result = 31 * result + annotations.hashCode();
+            return result;
+        }
+
+        @Override
+        public boolean equals(@MaybeNull Object other) {
             if (this == other) {
                 return true;
             } else if (other == null || getClass() != other.getClass()) {
@@ -517,15 +633,6 @@ public interface FieldDescription extends ByteCodeElement,
                     && type.equals(token.type)
                     && annotations.equals(token.annotations);
         }
-
-        @Override
-        public int hashCode() {
-            int result = name.hashCode();
-            result = 31 * result + modifiers;
-            result = 31 * result + type.hashCode();
-            result = 31 * result + annotations.hashCode();
-            return result;
-        }
     }
 
     /**
@@ -573,7 +680,15 @@ public interface FieldDescription extends ByteCodeElement,
         }
 
         @Override
-        public boolean equals(Object other) {
+        @CachedReturnPlugin.Enhance("hashCode")
+        public int hashCode() {
+            int result = name.hashCode();
+            result = 31 * result + type.hashCode();
+            return result;
+        }
+
+        @Override
+        public boolean equals(@MaybeNull Object other) {
             if (this == other) {
                 return true;
             } else if (!(other instanceof SignatureToken)) {
@@ -583,13 +698,6 @@ public interface FieldDescription extends ByteCodeElement,
             return name.equals(signatureToken.name) && type.equals(signatureToken.type);
         }
 
-        @Override
-        public int hashCode() {
-            int result = name.hashCode();
-            result = 31 * result + type.hashCode();
-            return result;
-        }
-
         @Override
         public String toString() {
             return type + " " + name;
