@@ -17,6 +17,7 @@ package net.bytebuddy.implementation;
 
 import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
 import net.bytebuddy.build.HashCodeAndEqualsPlugin;
+import net.bytebuddy.build.SafeVarargsPlugin;
 import net.bytebuddy.description.field.FieldDescription;
 import net.bytebuddy.description.method.MethodDescription;
 import net.bytebuddy.description.type.TypeDefinition;
@@ -227,7 +228,7 @@ public class EqualsMethod implements Implementation {
      * @return A new version of this equals method implementation that permits subclasses of the instrumented type to be equal to instances
      * of the instrumented type instead of requiring an exact match.
      */
-    public Implementation withSubclassEquality() {
+    public EqualsMethod withSubclassEquality() {
         return new EqualsMethod(superClassCheck, TypeCompatibilityCheck.SUBCLASS, ignored, nonNullable, identity, comparator);
     }
 
@@ -1030,6 +1031,7 @@ public class EqualsMethod implements Implementation {
          *
          * @param comparator All comparators to be applied in the application order.
          */
+        @SafeVarargsPlugin.Enhance
         @SuppressWarnings("unchecked") // In absence of @SafeVarargs
         protected CompoundComparator(Comparator<? super FieldDescription.InDefinedShape>... comparator) {
             this(Arrays.asList(comparator));
