@@ -1,6 +1,20 @@
+/*
+ * Copyright 2014 - Present Rafael Winterhalter
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 package net.bytebuddy.implementation.auxiliary;
 
-import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
 import net.bytebuddy.ByteBuddy;
 import net.bytebuddy.ClassFileVersion;
 import net.bytebuddy.build.HashCodeAndEqualsPlugin;
@@ -21,12 +35,12 @@ import net.bytebuddy.implementation.bytecode.member.MethodInvocation;
 import net.bytebuddy.implementation.bytecode.member.MethodReturn;
 import net.bytebuddy.implementation.bytecode.member.MethodVariableAccess;
 import net.bytebuddy.matcher.ElementMatchers;
+import net.bytebuddy.utility.RandomString;
 import org.objectweb.asm.MethodVisitor;
 import org.objectweb.asm.Opcodes;
 import org.objectweb.asm.Type;
 
 import java.io.Serializable;
-import java.util.List;
 
 import static net.bytebuddy.matcher.ElementMatchers.*;
 
@@ -94,7 +108,18 @@ public class TypeProxy implements AuxiliaryType {
         this.serializableProxy = serializableProxy;
     }
 
-    @Override
+    /**
+     * {@inheritDoc}
+     */
+    public String getSuffix() {
+        return RandomString.hashOf(proxiedType.hashCode())
+                + (ignoreFinalizer ? "I" : "0")
+                + (serializableProxy ? "S" : "0");
+    }
+
+    /**
+     * {@inheritDoc}
+     */
     public DynamicType make(String auxiliaryTypeName,
                             ClassFileVersion classFileVersion,
                             MethodAccessorFactory methodAccessorFactory) {
@@ -123,12 +148,11 @@ public class TypeProxy implements AuxiliaryType {
         /**
          * The stack manipulation that throws the abstract method error.
          */
-        private final StackManipulation implementation;
+        private final transient StackManipulation implementation;
 
         /**
          * Creates the singleton instance.
          */
-        @SuppressFBWarnings(value = "SE_BAD_FIELD_STORE", justification = "Fields of enumerations are never serialized")
         AbstractMethodErrorThrow() {
             TypeDescription abstractMethodError = TypeDescription.ForLoadedType.of(AbstractMethodError.class);
             MethodDescription constructor = abstractMethodError.getDeclaredMethods()
@@ -139,12 +163,16 @@ public class TypeProxy implements AuxiliaryType {
                     Throw.INSTANCE);
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public boolean isValid() {
             return implementation.isValid();
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public Size apply(MethodVisitor methodVisitor, Implementation.Context implementationContext) {
             return implementation.apply(methodVisitor, implementationContext);
         }
@@ -162,12 +190,16 @@ public class TypeProxy implements AuxiliaryType {
          */
         INSTANCE;
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public InstrumentedType prepare(InstrumentedType instrumentedType) {
             return instrumentedType;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public ByteCodeAppender appender(Target implementationTarget) {
             return new Appender(implementationTarget.getInstrumentedType());
         }
@@ -259,7 +291,9 @@ public class TypeProxy implements AuxiliaryType {
                 this.instrumentedType = instrumentedType;
             }
 
-            @Override
+            /**
+             * {@inheritDoc}
+             */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext, MethodDescription instrumentedMethod) {
                 methodVisitor.visitMethodInsn(Opcodes.INVOKESTATIC,
                         REFLECTION_FACTORY_INTERNAL_NAME,
@@ -325,7 +359,7 @@ public class TypeProxy implements AuxiliaryType {
              * Invokes the super method of the instrumented method.
              */
             SUPER_METHOD {
-                @Override
+                /** {@inheritDoc} */
                 public Implementation.SpecialMethodInvocation invoke(Implementation.Target implementationTarget,
                                                                      TypeDescription proxiedType,
                                                                      MethodDescription instrumentedMethod) {
@@ -337,13 +371,13 @@ public class TypeProxy implements AuxiliaryType {
              * Invokes the default method of the instrumented method if it exists and is not ambiguous.
              */
             DEFAULT_METHOD {
-                @Override
+                /** {@inheritDoc} */
                 public Implementation.SpecialMethodInvocation invoke(Implementation.Target implementationTarget,
                                                                      TypeDescription proxiedType,
                                                                      MethodDescription instrumentedMethod) {
                     return implementationTarget.invokeDefault(instrumentedMethod.asSignatureToken(), proxiedType);
                 }
-            };
+            }
         }
     }
 
@@ -353,23 +387,23 @@ public class TypeProxy implements AuxiliaryType {
      * All constructor parameters will be assigned their default values when this stack operation is applied.
      */
     @HashCodeAndEqualsPlugin.Enhance
-    public static class ForSuperMethodByConstructor implements StackManipulation {
+    public static class ForSuperMethodByConstructor extends StackManipulation.AbstractBase {
 
         /**
          * The type for the type proxy to subclass or implement.
          */
         private final TypeDescription proxiedType;
 
+        /**
+         * The constructor to invoke to create the proxy.
+         */
+        private final MethodDescription.InDefinedShape constructor;
+
         /**
          * The implementation target this type proxy is created for.
          */
         private final Implementation.Target implementationTarget;
 
-        /**
-         * The parameter types of the constructor that should be called.
-         */
-        private final List<TypeDescription> constructorParameters;
-
         /**
          * {@code true} if any finalizers should be ignored for the delegation.
          */
@@ -384,46 +418,44 @@ public class TypeProxy implements AuxiliaryType {
          * Creates a new stack operation for creating a type proxy by calling one of its constructors.
          *
          * @param proxiedType           The type for the type proxy to subclass or implement.
+         * @param constructor           The constructor to invoke to create the proxy.
          * @param implementationTarget  The implementation target this type proxy is created for.
-         * @param constructorParameters The parameter types of the constructor that should be called.
          * @param ignoreFinalizer       {@code true} if any finalizers should be ignored for the delegation.
          * @param serializableProxy     Determines if the proxy should be serializable.
          */
         public ForSuperMethodByConstructor(TypeDescription proxiedType,
+                                           MethodDescription.InDefinedShape constructor,
                                            Implementation.Target implementationTarget,
-                                           List<TypeDescription> constructorParameters,
                                            boolean ignoreFinalizer,
                                            boolean serializableProxy) {
             this.proxiedType = proxiedType;
+            this.constructor = constructor;
             this.implementationTarget = implementationTarget;
-            this.constructorParameters = constructorParameters;
             this.ignoreFinalizer = ignoreFinalizer;
             this.serializableProxy = serializableProxy;
         }
 
-        @Override
-        public boolean isValid() {
-            return true;
-        }
-
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public Size apply(MethodVisitor methodVisitor, Implementation.Context implementationContext) {
-            TypeDescription proxyType = implementationContext
-                    .register(new TypeProxy(proxiedType,
-                            implementationTarget,
-                            InvocationFactory.Default.SUPER_METHOD,
-                            ignoreFinalizer,
-                            serializableProxy));
-            StackManipulation[] constructorValue = new StackManipulation[constructorParameters.size()];
+            TypeDescription proxyType = implementationContext.register(new TypeProxy(proxiedType,
+                        implementationTarget,
+                        InvocationFactory.Default.SUPER_METHOD,
+                        ignoreFinalizer,
+                        serializableProxy));
+            StackManipulation[] constructorValue = new StackManipulation[constructor.getParameters().size()];
             int index = 0;
-            for (TypeDescription parameterType : constructorParameters) {
+            for (TypeDescription parameterType : constructor.getParameters().asTypeList().asErasures()) {
                 constructorValue[index++] = DefaultValue.of(parameterType);
             }
             return new Compound(
                     TypeCreation.of(proxyType),
                     Duplication.SINGLE,
                     new Compound(constructorValue),
-                    MethodInvocation.invoke(proxyType.getDeclaredMethods().filter(isConstructor().and(takesArguments(constructorParameters))).getOnly()),
+                    MethodInvocation.invoke(proxyType.getDeclaredMethods()
+                            .filter(isConstructor().and(takesArguments(constructor.getParameters().asTypeList().asErasures())))
+                            .getOnly()),
                     Duplication.SINGLE,
                     MethodVariableAccess.loadThis(),
                     FieldAccess.forField(proxyType.getDeclaredFields().filter((named(INSTANCE_FIELD))).getOnly()).write()
@@ -438,7 +470,7 @@ public class TypeProxy implements AuxiliaryType {
      * the instrumented type must lie on top of the operand stack.
      */
     @HashCodeAndEqualsPlugin.Enhance
-    public static class ForSuperMethodByReflectionFactory implements StackManipulation {
+    public static class ForSuperMethodByReflectionFactory extends StackManipulation.AbstractBase {
 
         /**
          * The type for which a proxy type is created.
@@ -478,12 +510,9 @@ public class TypeProxy implements AuxiliaryType {
             this.serializableProxy = serializableProxy;
         }
 
-        @Override
-        public boolean isValid() {
-            return true;
-        }
-
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public Size apply(MethodVisitor methodVisitor, Implementation.Context implementationContext) {
             TypeDescription proxyType = implementationContext.register(new TypeProxy(proxiedType,
                     implementationTarget,
@@ -504,7 +533,7 @@ public class TypeProxy implements AuxiliaryType {
      * a given interface and loads an instance of this proxy onto the operand stack.
      */
     @HashCodeAndEqualsPlugin.Enhance
-    public static class ForDefaultMethod implements StackManipulation {
+    public static class ForDefaultMethod extends StackManipulation.AbstractBase {
 
         /**
          * The proxied interface type.
@@ -536,12 +565,9 @@ public class TypeProxy implements AuxiliaryType {
             this.serializableProxy = serializableProxy;
         }
 
-        @Override
-        public boolean isValid() {
-            return true;
-        }
-
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public Size apply(MethodVisitor methodVisitor, Implementation.Context implementationContext) {
             TypeDescription proxyType = implementationContext.register(new TypeProxy(proxiedType,
                     implementationTarget,
@@ -579,14 +605,18 @@ public class TypeProxy implements AuxiliaryType {
             this.methodAccessorFactory = methodAccessorFactory;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public InstrumentedType prepare(InstrumentedType instrumentedType) {
             return instrumentedType.withField(new FieldDescription.Token(INSTANCE_FIELD,
                     Opcodes.ACC_PUBLIC | Opcodes.ACC_VOLATILE,
                     implementationTarget.getInstrumentedType().asGenericType()));
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public ByteCodeAppender appender(Target implementationTarget) {
             return new Appender(implementationTarget.getInstrumentedType());
         }
@@ -611,7 +641,9 @@ public class TypeProxy implements AuxiliaryType {
                 fieldLoadingInstruction = FieldAccess.forField(instrumentedType.getDeclaredFields().filter((named(INSTANCE_FIELD))).getOnly()).read();
             }
 
-            @Override
+            /**
+             * {@inheritDoc}
+             */
             public Size apply(MethodVisitor methodVisitor, Context implementationContext, MethodDescription instrumentedMethod) {
                 SpecialMethodInvocation specialMethodInvocation = invocationFactory.invoke(implementationTarget, proxiedType, instrumentedMethod);
                 StackManipulation.Size size = (specialMethodInvocation.isValid()
@@ -649,12 +681,16 @@ public class TypeProxy implements AuxiliaryType {
                     this.specialMethodInvocation = specialMethodInvocation;
                 }
 
-                @Override
+                /**
+                 * {@inheritDoc}
+                 */
                 public boolean isValid() {
                     return specialMethodInvocation.isValid();
                 }
 
-                @Override
+                /**
+                 * {@inheritDoc}
+                 */
                 public Size apply(MethodVisitor methodVisitor, Implementation.Context implementationContext) {
                     MethodDescription.InDefinedShape proxyMethod = methodAccessorFactory.registerAccessorFor(specialMethodInvocation, MethodAccessorFactory.AccessType.DEFAULT);
                     return new StackManipulation.Compound(
