@@ -1,3 +1,18 @@
+/*
+ * Copyright 2014 - Present Rafael Winterhalter
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 package net.bytebuddy.implementation.bytecode.member;
 
 import net.bytebuddy.ClassFileVersion;
@@ -7,9 +22,9 @@ import net.bytebuddy.description.type.TypeDescription;
 import net.bytebuddy.description.type.TypeList;
 import net.bytebuddy.implementation.Implementation;
 import net.bytebuddy.implementation.bytecode.StackManipulation;
-import net.bytebuddy.implementation.bytecode.StackSize;
 import net.bytebuddy.implementation.bytecode.assign.TypeCasting;
-import org.objectweb.asm.Handle;
+import net.bytebuddy.utility.JavaConstant;
+import net.bytebuddy.utility.JavaType;
 import org.objectweb.asm.MethodVisitor;
 import org.objectweb.asm.Opcodes;
 
@@ -128,6 +143,17 @@ public enum MethodInvocation {
                 : OfGenericMethod.of(methodDescription, invoke(declaredMethod));
     }
 
+    /**
+     * Returns a method invocation of {@code java.lang.invoke.MethodHandles#lookup()}.
+     *
+     * @return A method invocation for resolving the current lookup.
+     */
+    public static StackManipulation lookup() {
+        return invoke(new MethodDescription.Latent(JavaType.METHOD_HANDLES.getTypeStub(), new MethodDescription.Token("lookup",
+                Opcodes.ACC_STATIC | Opcodes.ACC_PUBLIC,
+                JavaType.METHOD_HANDLES_LOOKUP.getTypeStub().asGenericType())));
+    }
+
     /**
      * An illegal implicit method invocation.
      */
@@ -138,35 +164,47 @@ public enum MethodInvocation {
          */
         INSTANCE;
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public StackManipulation virtual(TypeDescription invocationTarget) {
             return Illegal.INSTANCE;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public StackManipulation special(TypeDescription invocationTarget) {
             return Illegal.INSTANCE;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public StackManipulation dynamic(String methodName,
                                          TypeDescription returnType,
                                          List<? extends TypeDescription> methodType,
-                                         List<?> arguments) {
+                                         List<? extends JavaConstant> arguments) {
             return Illegal.INSTANCE;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public StackManipulation onHandle(HandleType type) {
             return Illegal.INSTANCE;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public boolean isValid() {
             return false;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public Size apply(MethodVisitor methodVisitor, Implementation.Context implementationContext) {
             return Illegal.INSTANCE.apply(methodVisitor, implementationContext);
         }
@@ -208,7 +246,7 @@ public enum MethodInvocation {
         StackManipulation dynamic(String methodName,
                                   TypeDescription returnType,
                                   List<? extends TypeDescription> methodType,
-                                  List<?> arguments);
+                                  List<? extends JavaConstant> arguments);
 
         /**
          * Invokes the method via a {@code MethodHandle}.
@@ -257,32 +295,44 @@ public enum MethodInvocation {
             return new OfGenericMethod(methodDescription.getReturnType().asErasure(), invocation);
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public StackManipulation virtual(TypeDescription invocationTarget) {
             return new StackManipulation.Compound(invocation.virtual(invocationTarget), TypeCasting.to(targetType));
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public StackManipulation special(TypeDescription invocationTarget) {
             return new StackManipulation.Compound(invocation.special(invocationTarget), TypeCasting.to(targetType));
         }
 
-        @Override
-        public StackManipulation dynamic(String methodName, TypeDescription returnType, List<? extends TypeDescription> methodType, List<?> arguments) {
+        /**
+         * {@inheritDoc}
+         */
+        public StackManipulation dynamic(String methodName, TypeDescription returnType, List<? extends TypeDescription> methodType, List<? extends JavaConstant> arguments) {
             return invocation.dynamic(methodName, returnType, methodType, arguments);
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public StackManipulation onHandle(HandleType type) {
             return new Compound(invocation.onHandle(type), TypeCasting.to(targetType));
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public boolean isValid() {
             return invocation.isValid();
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public Size apply(MethodVisitor methodVisitor, Implementation.Context implementationContext) {
             return new Compound(invocation, TypeCasting.to(targetType)).apply(methodVisitor, implementationContext);
         }
@@ -292,7 +342,7 @@ public enum MethodInvocation {
      * An implementation of a method invoking stack manipulation.
      */
     @HashCodeAndEqualsPlugin.Enhance(includeSyntheticFields = true)
-    protected class Invocation implements WithImplicitInvocationTargetType {
+    protected class Invocation extends StackManipulation.AbstractBase implements WithImplicitInvocationTargetType {
 
         /**
          * The method to be invoked.
@@ -324,12 +374,9 @@ public enum MethodInvocation {
             this.methodDescription = methodDescription;
         }
 
-        @Override
-        public boolean isValid() {
-            return true;
-        }
-
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public Size apply(MethodVisitor methodVisitor, Implementation.Context implementationContext) {
             methodVisitor.visitMethodInsn(opcode == legacyOpcode || implementationContext.getClassFileVersion().isAtLeast(ClassFileVersion.JAVA_V11)
                             ? opcode
@@ -342,7 +389,9 @@ public enum MethodInvocation {
             return new Size(returnValueSize - parameterSize, Math.max(0, returnValueSize - parameterSize));
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public StackManipulation virtual(TypeDescription invocationTarget) {
             if (methodDescription.isConstructor() || methodDescription.isStatic()) {
                 return Illegal.INSTANCE;
@@ -359,113 +408,42 @@ public enum MethodInvocation {
             }
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public StackManipulation special(TypeDescription invocationTarget) {
             return methodDescription.isSpecializableFor(invocationTarget)
                     ? SPECIAL.new Invocation(methodDescription, invocationTarget)
                     : Illegal.INSTANCE;
         }
 
-        @Override
-        public StackManipulation dynamic(String methodName,
+        /**
+         * {@inheritDoc}
+         */
+        public StackManipulation dynamic(String name,
                                          TypeDescription returnType,
                                          List<? extends TypeDescription> methodType,
-                                         List<?> arguments) {
-            return methodDescription.isInvokeBootstrap()
-                    ? new DynamicInvocation(methodName, returnType, new TypeList.Explicit(methodType), methodDescription.asDefined(), arguments)
-                    : Illegal.INSTANCE;
+                                         List<? extends JavaConstant> arguments) {
+            return methodDescription.isInvokeBootstrap(TypeList.Explicit.of(arguments)) ? new Invokedynamic(name,
+                    JavaConstant.MethodType.of(returnType, methodType),
+                    JavaConstant.MethodHandle.of(methodDescription),
+                    arguments) : Illegal.INSTANCE;
         }
 
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public StackManipulation onHandle(HandleType type) {
             return new HandleInvocation(methodDescription, type);
         }
     }
 
-    /**
-     * Performs a dynamic method invocation of the given method.
-     */
-    @HashCodeAndEqualsPlugin.Enhance(includeSyntheticFields = true)
-    protected class DynamicInvocation implements StackManipulation {
-
-        /**
-         * The internal name of the method that is to be bootstrapped.
-         */
-        private final String methodName;
-
-        /**
-         * The return type of the method to be bootstrapped.
-         */
-        private final TypeDescription returnType;
-
-        /**
-         * The parameter types of the method to be bootstrapped.
-         */
-        private final List<? extends TypeDescription> parameterTypes;
-
-        /**
-         * The bootstrap method.
-         */
-        private final MethodDescription.InDefinedShape bootstrapMethod;
-
-        /**
-         * The list of arguments to be handed over to the bootstrap method.
-         */
-        private final List<?> arguments;
-
-        /**
-         * Creates a new dynamic method invocation.
-         *
-         * @param methodName      The internal name of the method that is to be bootstrapped.
-         * @param returnType      The return type of the method to be bootstrapped.
-         * @param parameterTypes  The type of the parameters to be bootstrapped.
-         * @param bootstrapMethod The bootstrap method.
-         * @param arguments       The list of arguments to be handed over to the bootstrap method.
-         */
-        public DynamicInvocation(String methodName,
-                                 TypeDescription returnType,
-                                 List<? extends TypeDescription> parameterTypes,
-                                 MethodDescription.InDefinedShape bootstrapMethod,
-                                 List<?> arguments) {
-            this.methodName = methodName;
-            this.returnType = returnType;
-            this.parameterTypes = parameterTypes;
-            this.bootstrapMethod = bootstrapMethod;
-            this.arguments = arguments;
-        }
-
-        @Override
-        public boolean isValid() {
-            return true;
-        }
-
-        @Override
-        public Size apply(MethodVisitor methodVisitor, Implementation.Context implementationContext) {
-            StringBuilder stringBuilder = new StringBuilder("(");
-            for (TypeDescription parameterType : parameterTypes) {
-                stringBuilder.append(parameterType.getDescriptor());
-            }
-            String methodDescriptor = stringBuilder.append(')').append(returnType.getDescriptor()).toString();
-            methodVisitor.visitInvokeDynamicInsn(methodName,
-                    methodDescriptor,
-                    new Handle(handle == legacyHandle || implementationContext.getClassFileVersion().isAtLeast(ClassFileVersion.JAVA_V11)
-                            ? handle
-                            : legacyHandle,
-                            bootstrapMethod.getDeclaringType().getInternalName(),
-                            bootstrapMethod.getInternalName(),
-                            bootstrapMethod.getDescriptor(),
-                            bootstrapMethod.getDeclaringType().isInterface()),
-                    arguments.toArray(new Object[arguments.size()]));
-            int stackSize = returnType.getStackSize().getSize() - StackSize.of(parameterTypes);
-            return new Size(stackSize, Math.max(stackSize, 0));
-        }
-    }
 
     /**
      * Performs a method invocation on a method handle with a polymorphic type signature.
      */
     @HashCodeAndEqualsPlugin.Enhance
-    protected static class HandleInvocation implements StackManipulation {
+    protected static class HandleInvocation extends StackManipulation.AbstractBase {
 
         /**
          * The internal name of the method handle type.
@@ -493,12 +471,9 @@ public enum MethodInvocation {
             this.type = type;
         }
 
-        @Override
-        public boolean isValid() {
-            return true;
-        }
-
-        @Override
+        /**
+         * {@inheritDoc}
+         */
         public Size apply(MethodVisitor methodVisitor, Implementation.Context implementationContext) {
             methodVisitor.visitMethodInsn(Opcodes.INVOKEVIRTUAL,
                     METHOD_HANDLE,
