@@ -2,13 +2,16 @@ package net.bytebuddy.agent.builder;
 
 import net.bytebuddy.build.Plugin;
 import net.bytebuddy.description.type.TypeDescription;
+import net.bytebuddy.dynamic.ClassFileLocator;
 import net.bytebuddy.dynamic.DynamicType;
-import net.bytebuddy.test.utility.MockitoRule;
 import net.bytebuddy.utility.JavaModule;
 import org.junit.Rule;
 import org.junit.Test;
-import org.junit.rules.TestRule;
+import org.junit.rules.MethodRule;
 import org.mockito.Mock;
+import org.mockito.junit.MockitoJUnit;
+
+import java.security.ProtectionDomain;
 
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.MatcherAssert.assertThat;
@@ -17,7 +20,7 @@ import static org.mockito.Mockito.*;
 public class AgentBuilderTransformerForBuildPluginTest {
 
     @Rule
-    public TestRule mockitoRule = new MockitoRule(this);
+    public MethodRule mockitoRule = MockitoJUnit.rule().silent();
 
     @Mock
     private Plugin plugin;
@@ -34,12 +37,15 @@ public class AgentBuilderTransformerForBuildPluginTest {
     @Mock
     private JavaModule module;
 
+    @Mock
+    private ProtectionDomain protectionDomain;
+
     @Test
     @SuppressWarnings("unchecked")
     public void testApplication() throws Exception {
-        when(plugin.apply(builder, typeDescription)).thenReturn((DynamicType.Builder) result);
-        assertThat(new AgentBuilder.Transformer.ForBuildPlugin(plugin).transform(builder, typeDescription, classLoader, module), is((DynamicType.Builder) result));
-        verify(plugin).apply(builder, typeDescription);
+        when(plugin.apply(eq(builder), eq(typeDescription), any(ClassFileLocator.ForClassLoader.class))).thenReturn((DynamicType.Builder) result);
+        assertThat(new AgentBuilder.Transformer.ForBuildPlugin(plugin).transform(builder, typeDescription, classLoader, module, protectionDomain), is((DynamicType.Builder) result));
+        verify(plugin).apply(eq(builder), eq(typeDescription), any(ClassFileLocator.ForClassLoader.class));
         verifyNoMoreInteractions(plugin);
     }
 }
