@@ -12,6 +12,7 @@ import org.objectweb.asm.ClassVisitor;
 
 import java.io.File;
 import java.io.FileOutputStream;
+import java.io.OutputStream;
 import java.util.jar.JarEntry;
 import java.util.jar.JarFile;
 import java.util.jar.JarOutputStream;
@@ -42,15 +43,17 @@ public class ClassFileLocatorForJarFileTest {
 
     @Test
     public void testSuccessfulLocation() throws Exception {
-        JarOutputStream jarOutputStream = new JarOutputStream(new FileOutputStream(file));
+        OutputStream outputStream = new FileOutputStream(file);
         try {
+            JarOutputStream jarOutputStream = new JarOutputStream(outputStream);
             JarEntry jarEntry = new JarEntry(FOO + "/" + BAR + ClassFileLocator.CLASS_FILE_EXTENSION);
             jarOutputStream.putNextEntry(jarEntry);
             jarOutputStream.write(VALUE);
             jarOutputStream.write(VALUE * 2);
             jarOutputStream.closeEntry();
-        } finally {
             jarOutputStream.close();
+        } finally {
+            outputStream.close();
         }
         JarFile jarFile = new JarFile(file);
         try {
@@ -88,14 +91,16 @@ public class ClassFileLocatorForJarFileTest {
 
     @Test
     public void testNonSuccessfulLocation() throws Exception {
-        JarOutputStream jarOutputStream = new JarOutputStream(new FileOutputStream(file));
+        OutputStream outputStream = new FileOutputStream(file);
         try {
+            JarOutputStream jarOutputStream = new JarOutputStream(outputStream);
             JarEntry jarEntry = new JarEntry("noop.class");
             jarOutputStream.putNextEntry(jarEntry);
             jarOutputStream.write(VALUE);
             jarOutputStream.closeEntry();
-        } finally {
             jarOutputStream.close();
+        } finally {
+            outputStream.close();
         }
         JarFile jarFile = new JarFile(file);
         try {
@@ -127,14 +132,16 @@ public class ClassFileLocatorForJarFileTest {
         Manifest manifest = new Manifest();
         manifest.getMainAttributes().putValue("Manifest-Version", "1.0");
         manifest.getMainAttributes().putValue("Multi-Release", "true");
-        JarOutputStream jarOutputStream = new JarOutputStream(new FileOutputStream(file), manifest);
+        OutputStream outputStream = new FileOutputStream(file);
         try {
+            JarOutputStream jarOutputStream = new JarOutputStream(outputStream, manifest);
             jarOutputStream.putNextEntry(new JarEntry("META-INF/versions/11/" + FOO + "/" + BAR + ClassFileLocator.CLASS_FILE_EXTENSION));
             jarOutputStream.write(VALUE);
             jarOutputStream.write(VALUE * 2);
             jarOutputStream.closeEntry();
-        } finally {
             jarOutputStream.close();
+        } finally {
+            outputStream.close();
         }
         ClassFileLocator classFileLocator = ClassFileLocator.ForJarFile.of(file, ClassFileVersion.JAVA_V11);
         ClassFileLocator.Resolution resolution = classFileLocator.locate(FOO + "." + BAR);
