@@ -24,6 +24,7 @@ public class SetAccessibleActionTest {
         Object bar, qux;
     }
 
+    @SuppressWarnings("deprecation")
     private static class AccessibleObjectSpy extends AccessibleObject {
 
         private final AccessibleObject accessibleObject;
@@ -34,7 +35,6 @@ public class SetAccessibleActionTest {
             this.accessibleObject = accessibleObject;
         }
 
-        @Override
         public void setAccessible(boolean flag) {
             accessible = flag;
             accessibleObject.setAccessible(flag);
